import { Shape } from '@antv/g';
import { Module } from 'mana-syringe';
import { CanvasPickerPlugin, PointInPathPickerFactory } from './CanvasPickerPlugin';
import { isPointInPath as CirclePicker } from './Circle';
import { isPointInPath as EllipsePicker } from './Ellipse';
import { isPointInPath as LinePicker } from './Line';
import { isPointInPath as PolylinePicker } from './Polyline';
import { isPointInPath as PathPicker } from './Path';
import { isPointInPath as RectPicker } from './Rect';
import { isPointInPath as PolygonPicker } from './Polygon';
var containerModule = Module(function (register) {
  register({
    token: PointInPathPickerFactory,
    useFactory: function useFactory(context) {
      return function (tagName) {
        if (tagName === Shape.CIRCLE) {
          return CirclePicker;
        } else if (tagName === Shape.ELLIPSE) {
          return EllipsePicker;
        } else if (tagName === Shape.RECT) {
          return RectPicker;
        } else if (tagName === Shape.LINE) {
          return LinePicker;
        } else if (tagName === Shape.POLYLINE) {
          return PolylinePicker;
        } else if (tagName === Shape.POLYGON) {
          return PolygonPicker;
        } else if (tagName === Shape.PATH) {
          return PathPicker;
        } else if (tagName === Shape.IMAGE || tagName === Shape.TEXT // tagName === Shape.GROUP
        ) {
          return function () {
            return true;
          };
        }

        return null;
      };
    }
  });
  register(CanvasPickerPlugin);
});

var Plugin =
/** @class */
function () {
  function Plugin() {}

  Plugin.prototype.init = function (container) {
    container.load(containerModule, true);
  };

  Plugin.prototype.destroy = function (container) {
    container.unload(containerModule);
  };

  return Plugin;
}();

export { Plugin };